package ssw.mj;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;

import ssw.mj.codegen.Code;
 
/** 
 * <code>Compiler</code> is the driver for the MicroJava-Compiler.
 * 
 * Execute<br>
 * <code>java ssw.mj.Compiler &lt;<i>MJ-Source-Filename</i>&gt;</code><br>
 * to start compililation.
 * 
 * @author AW
 */
public class Compiler {

	private static String objFileName(String s) {
		int i = s.lastIndexOf('.');
		if (i < 0) return s + ".obj";
		else return s.substring(0, i) + ".obj";
	}
	
	public static void main(String[] args) {
		// --- get the filename
		if (args.length < 1) { 
			System.out.println("usage: java Compiler filename.mj"); 
			return;
		}
		String filename = args[0];
		String output = objFileName(filename);
		
		try {
			System.out.println("-----------------------------------");
			Scanner.init(new FileReader(filename));
			System.out.println(Messages.getString("PARSING") + filename);
			Parser.parse();
			int errCnt = Parser.Errors.count();
			switch (errCnt) {
				case 0:	
					System.out.println(Messages.getString("NO_ERRORS")); 
					System.out.println(Messages.getString("GEN_OBJ_FILE"));
					Code.write(new BufferedOutputStream(
										new FileOutputStream(output)));
					System.out.println(Messages.getString("DONE"));
					break;
				case 1: System.out.println(Messages.getString("ONE_ERROR")); break;
				default: 
					System.out.println("\n" + errCnt + Messages.getString("N_ERRORS")); 
					break;
			}
		} catch (IOException ioe) {
		  System.out.println(Messages.getString("FILE") + filename + 
		                     Messages.getString("FILE_NOT_FOUND"));
		}		
	}
}
