package ssw.mj.codegen;

import ssw.mj.Parser;
import ssw.mj.symtab.Obj;
import ssw.mj.symtab.Struct;
import ssw.mj.symtab.Tab;

/**
 * MicroJava Code Item.
 * An item stores the attributes of an operand during code generation.
 * 
 * @author AW
 */
public class Item {
	// codes for the item kinds
	public static final int
		Con = 0, Local = 1, Static = 2, Stack = 3, Fld = 4, Elem = 5, Meth = 6;

	public int kind;              // Con, Local, Static, Stack, Fld, Elem, Meth
	public Struct type;           // item type
	public int val;               // Con: value
	public int adr;               // Local, Static, Fld, Meth: offset
	public Obj obj;               // Meth: method object from symbol table

	public Item (Obj o) {
		type = o.type; val = o.val; adr = o.adr;
		switch (o.kind) {
			case Obj.Con:  kind = Con; break;
			case Obj.Var:  if (o.level == 0) kind = Static; else kind = Local; break;
			case Obj.Meth: kind = Meth; obj = o; break;
			default: Parser.Errors.error("CREATE_ITEM");
		}
	}

	public Item (int kind, int x, Struct type) {
		this.kind = kind; this.type = type;
		switch (kind) {
			case Con: val = x; break;
			case Local: case Static: case Fld: case Meth: adr = x; break;
		}
	}

	// special constructor for integer constants
	public Item (int x) { kind = Con; type = Tab.intType; val = x; }
}
